# coding: utf-8

# (C) Copyright IBM Corp. 2016, 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
The IBM Watson&trade; Visual Recognition service uses deep learning algorithms to identify
scenes and objects in images that you upload to the service. You can create and train a
custom classifier to identify subjects that suit your needs.
"""

import json
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from .common import get_sdk_headers
from datetime import datetime
from enum import Enum
from ibm_cloud_sdk_core import BaseService
from ibm_cloud_sdk_core import DetailedResponse
from ibm_cloud_sdk_core import datetime_to_string, string_to_datetime
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment
from os.path import basename
from typing import BinaryIO
from typing import Dict
from typing import List

##############################################################################
# Service
##############################################################################


class VisualRecognitionV3(BaseService):
    """The Visual Recognition V3 service."""

    DEFAULT_SERVICE_URL = 'https://api.us-south.visual-recognition.watson.cloud.ibm.com'
    DEFAULT_SERVICE_NAME = 'visual_recognition'

    def __init__(
        self,
        version: str,
        authenticator: Authenticator = None,
        service_name: str = DEFAULT_SERVICE_NAME,
    ) -> None:
        """
        Construct a new client for the Visual Recognition service.

        :param str version: The API version date to use with the service, in
               "YYYY-MM-DD" format. Whenever the API is changed in a backwards
               incompatible way, a new minor version of the API is released.
               The service uses the API version for the date you specify, or
               the most recent version before that date. Note that you should
               not programmatically specify the current date at runtime, in
               case the API has been updated since your application's release.
               Instead, specify a version date that is compatible with your
               application, and don't change it until your application is
               ready for a later version.

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/master/README.md
               about initializing the authenticator of your choice.
        """
        if not authenticator:
            authenticator = get_authenticator_from_environment(service_name)
        BaseService.__init__(self,
                             service_url=self.DEFAULT_SERVICE_URL,
                             authenticator=authenticator,
                             disable_ssl_verification=False)
        self.version = version
        self.configure_service(service_name)

    #########################
    # General
    #########################

    def classify(self,
                 *,
                 images_file: BinaryIO = None,
                 images_filename: str = None,
                 images_file_content_type: str = None,
                 url: str = None,
                 threshold: float = None,
                 owners: str = None,
                 classifier_ids: str = None,
                 accept_language: str = None,
                 **kwargs) -> 'DetailedResponse':
        """
        Classify images.

        Classify images with built-in or custom classifiers.

        :param TextIO images_file: (optional) An image file (.gif, .jpg, .png,
               .tif) or .zip file with images. Maximum image size is 10 MB. Include no
               more than 20 images and limit the .zip file to 100 MB. Encode the image and
               .zip file names in UTF-8 if they contain non-ASCII characters. The service
               assumes UTF-8 encoding if it encounters non-ASCII characters.
               You can also include an image with the **url** parameter.
        :param str images_filename: (optional) The filename for images_file.
        :param str images_file_content_type: (optional) The content type of
               images_file.
        :param str url: (optional) The URL of an image (.gif, .jpg, .png, .tif) to
               analyze. The minimum recommended pixel density is 32X32 pixels, but the
               service tends to perform better with images that are at least 224 x 224
               pixels. The maximum image size is 10 MB.
               You can also include images with the **images_file** parameter.
        :param float threshold: (optional) The minimum score a class must have to
               be displayed in the response. Set the threshold to `0.0` to return all
               identified classes.
        :param List[str] owners: (optional) The categories of classifiers to apply.
               The **classifier_ids** parameter overrides **owners**, so make sure that
               **classifier_ids** is empty.
               - Use `IBM` to classify against the `default` general classifier. You get
               the same result if both **classifier_ids** and **owners** parameters are
               empty.
               - Use `me` to classify against all your custom classifiers. However, for
               better performance use **classifier_ids** to specify the specific custom
               classifiers to apply.
               - Use both `IBM` and `me` to analyze the image against both classifier
               categories.
        :param List[str] classifier_ids: (optional) Which classifiers to apply.
               Overrides the **owners** parameter. You can specify both custom and
               built-in classifier IDs. The built-in `default` classifier is used if both
               **classifier_ids** and **owners** parameters are empty.
               The following built-in classifier IDs require no training:
               - `default`: Returns classes from thousands of general tags.
               - `food`: Enhances specificity and accuracy for images of food items.
               - `explicit`: Evaluates whether the image might be pornographic.
        :param str accept_language: (optional) The desired language of parts of the
               response. See the response for details.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        headers = {'Accept-Language': accept_language}
        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V3',
                                      operation_id='classify')
        headers.update(sdk_headers)

        params = {'version': self.version}

        form_data = []
        if images_file:
            if not images_filename and hasattr(images_file, 'name'):
                images_filename = basename(images_file.name)
            if not images_filename:
                raise ValueError('images_filename must be provided')
            form_data.append(('images_file', (images_filename, images_file,
                                              images_file_content_type or
                                              'application/octet-stream')))
        if url:
            url = str(url)
            form_data.append(('url', (None, url, 'text/plain')))
        if threshold:
            threshold = str(threshold)
            form_data.append(('threshold', (None, threshold, 'text/plain')))
        if owners:
            owners = self._convert_list(owners)
            form_data.append(('owners', (None, owners, 'text/plain')))
        if classifier_ids:
            classifier_ids = self._convert_list(classifier_ids)
            form_data.append(
                ('classifier_ids', (None, classifier_ids, 'text/plain')))

        url = '/v3/classify'
        request = self.prepare_request(method='POST',
                                       url=url,
                                       headers=headers,
                                       params=params,
                                       files=form_data)

        response = self.send(request)
        return response

    #########################
    # Custom
    #########################

    def create_classifier(self,
                          name: str,
                          positive_examples: BinaryIO,
                          *,
                          negative_examples: BinaryIO = None,
                          negative_examples_filename: str = None,
                          **kwargs) -> 'DetailedResponse':
        """
        Create a classifier.

        Train a new multi-faceted classifier on the uploaded image data. Create your
        custom classifier with positive or negative example training images. Include at
        least two sets of examples, either two positive example files or one positive and
        one negative file. You can upload a maximum of 256 MB per call.
        **Tips when creating:**
        - If you set the **X-Watson-Learning-Opt-Out** header parameter to `true` when you
        create a classifier, the example training images are not stored. Save your
        training images locally. For more information, see [Data
        collection](#data-collection).
        - Encode all names in UTF-8 if they contain non-ASCII characters (.zip and image
        file names, and classifier and class names). The service assumes UTF-8 encoding if
        it encounters non-ASCII characters.

        :param str name: The name of the new classifier. Encode special characters
               in UTF-8.
        :param dict positive_examples: A dictionary that contains the value for
               each classname. The value is a .zip file of images that depict the visual
               subject of a class in the new classifier. You can include more than one
               positive example file in a call.
               Specify the parameter name by appending `_positive_examples` to the class
               name. For example, `goldenretriever_positive_examples` creates the class
               **goldenretriever**. The string cannot contain the following characters:
               ``$ * - { } \ | / ' " ` [ ]``.
               Include at least 10 images in .jpg or .png format. The minimum recommended
               image resolution is 32X32 pixels. The maximum number of images is 10,000
               images or 100 MB per .zip file.
               Encode special characters in the file name in UTF-8.
        :param TextIO negative_examples: (optional) A .zip file of images that do
               not depict the visual subject of any of the classes of the new classifier.
               Must contain a minimum of 10 images.
               Encode special characters in the file name in UTF-8.
        :param str negative_examples_filename: (optional) The filename for
               negative_examples.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if name is None:
            raise ValueError('name must be provided')
        if not positive_examples:
            raise ValueError('positive_examples must be provided')

        headers = {}
        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V3',
                                      operation_id='create_classifier')
        headers.update(sdk_headers)

        params = {'version': self.version}

        form_data = []
        name = str(name)
        form_data.append(('name', (None, name, 'text/plain')))
        for key in positive_examples.keys():
            part_name = '%s_positive_examples' % (key)
            value = positive_examples[key]
            if hasattr(value, 'name'):
                filename = basename(value.name)
                form_data.append(
                    (part_name, (filename, value, 'application/octet-stream')))
        if negative_examples:
            if not negative_examples_filename and hasattr(
                    negative_examples, 'name'):
                negative_examples_filename = basename(negative_examples.name)
            if not negative_examples_filename:
                raise ValueError('negative_examples_filename must be provided')
            form_data.append(('negative_examples',
                              (negative_examples_filename, negative_examples,
                               'application/octet-stream')))

        url = '/v3/classifiers'
        request = self.prepare_request(method='POST',
                                       url=url,
                                       headers=headers,
                                       params=params,
                                       files=form_data)

        response = self.send(request)
        return response

    def list_classifiers(self,
                         *,
                         verbose: bool = None,
                         **kwargs) -> 'DetailedResponse':
        """
        Retrieve a list of classifiers.

        :param bool verbose: (optional) Specify `true` to return details about the
               classifiers. Omit this parameter to return a brief list of classifiers.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        headers = {}
        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V3',
                                      operation_id='list_classifiers')
        headers.update(sdk_headers)

        params = {'version': self.version, 'verbose': verbose}

        url = '/v3/classifiers'
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers,
                                       params=params)

        response = self.send(request)
        return response

    def get_classifier(self, classifier_id: str,
                       **kwargs) -> 'DetailedResponse':
        """
        Retrieve classifier details.

        Retrieve information about a custom classifier.

        :param str classifier_id: The ID of the classifier.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if classifier_id is None:
            raise ValueError('classifier_id must be provided')

        headers = {}
        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V3',
                                      operation_id='get_classifier')
        headers.update(sdk_headers)

        params = {'version': self.version}

        url = '/v3/classifiers/{0}'.format(
            *self._encode_path_vars(classifier_id))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers,
                                       params=params)

        response = self.send(request)
        return response

    def update_classifier(self,
                          classifier_id: str,
                          *,
                          positive_examples: BinaryIO = {},
                          negative_examples: BinaryIO = None,
                          negative_examples_filename: str = None,
                          **kwargs) -> 'DetailedResponse':
        """
        Update a classifier.

        Update a custom classifier by adding new positive or negative classes or by adding
        new images to existing classes. You must supply at least one set of positive or
        negative examples. For details, see [Updating custom
        classifiers](https://cloud.ibm.com/docs/visual-recognition?topic=visual-recognition-customizing#updating-custom-classifiers).
        Encode all names in UTF-8 if they contain non-ASCII characters (.zip and image
        file names, and classifier and class names). The service assumes UTF-8 encoding if
        it encounters non-ASCII characters.
        **Tips about retraining:**
        - You can't update the classifier if the **X-Watson-Learning-Opt-Out** header
        parameter was set to `true` when the classifier was created. Training images are
        not stored in that case. Instead, create another classifier. For more information,
        see [Data collection](#data-collection).
        - Don't make retraining calls on a classifier until the status is ready. When you
        submit retraining requests in parallel, the last request overwrites the previous
        requests. The `retrained` property shows the last time the classifier retraining
        finished.

        :param str classifier_id: The ID of the classifier.
        :param dict positive_examples: (optional) A dictionary that contains the
               value for each classname. The value is a .zip file of images that depict
               the visual subject of a class in the classifier. The positive examples
               create or update classes in the classifier. You can include more than one
               positive example file in a call.
               Specify the parameter name by appending `_positive_examples` to the class
               name. For example, `goldenretriever_positive_examples` creates the class
               `goldenretriever`. The string cannot contain the following characters: ``$
               * - { } \ | / ' " ` [ ]``.
               Include at least 10 images in .jpg or .png format. The minimum recommended
               image resolution is 32X32 pixels. The maximum number of images is 10,000
               images or 100 MB per .zip file.
               Encode special characters in the file name in UTF-8.
        :param TextIO negative_examples: (optional) A .zip file of images that do
               not depict the visual subject of any of the classes of the new classifier.
               Must contain a minimum of 10 images.
               Encode special characters in the file name in UTF-8.
        :param str negative_examples_filename: (optional) The filename for
               negative_examples.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if classifier_id is None:
            raise ValueError('classifier_id must be provided')

        headers = {}
        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V3',
                                      operation_id='update_classifier')
        headers.update(sdk_headers)

        params = {'version': self.version}

        form_data = []
        for key in positive_examples.keys():
            part_name = '%s_positive_examples' % (key)
            value = positive_examples[key]
            if hasattr(value, 'name'):
                filename = basename(value.name)
                form_data.append(
                    (part_name, (filename, value, 'application/octet-stream')))
        if negative_examples:
            if not negative_examples_filename and hasattr(
                    negative_examples, 'name'):
                negative_examples_filename = basename(negative_examples.name)
            if not negative_examples_filename:
                raise ValueError('negative_examples_filename must be provided')
            form_data.append(('negative_examples',
                              (negative_examples_filename, negative_examples,
                               'application/octet-stream')))

        url = '/v3/classifiers/{0}'.format(
            *self._encode_path_vars(classifier_id))
        request = self.prepare_request(method='POST',
                                       url=url,
                                       headers=headers,
                                       params=params,
                                       files=form_data)

        response = self.send(request)
        return response

    def delete_classifier(self, classifier_id: str,
                          **kwargs) -> 'DetailedResponse':
        """
        Delete a classifier.

        :param str classifier_id: The ID of the classifier.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if classifier_id is None:
            raise ValueError('classifier_id must be provided')

        headers = {}
        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V3',
                                      operation_id='delete_classifier')
        headers.update(sdk_headers)

        params = {'version': self.version}

        url = '/v3/classifiers/{0}'.format(
            *self._encode_path_vars(classifier_id))
        request = self.prepare_request(method='DELETE',
                                       url=url,
                                       headers=headers,
                                       params=params)

        response = self.send(request)
        return response

    #########################
    # Core ML
    #########################

    def get_core_ml_model(self, classifier_id: str,
                          **kwargs) -> 'DetailedResponse':
        """
        Retrieve a Core ML model of a classifier.

        Download a Core ML model file (.mlmodel) of a custom classifier that returns
        <tt>"core_ml_enabled": true</tt> in the classifier details.

        :param str classifier_id: The ID of the classifier.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if classifier_id is None:
            raise ValueError('classifier_id must be provided')

        headers = {}
        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V3',
                                      operation_id='get_core_ml_model')
        headers.update(sdk_headers)

        params = {'version': self.version}

        url = '/v3/classifiers/{0}/core_ml_model'.format(
            *self._encode_path_vars(classifier_id))
        request = self.prepare_request(method='GET',
                                       url=url,
                                       headers=headers,
                                       params=params)

        response = self.send(request)
        return response

    #########################
    # User data
    #########################

    def delete_user_data(self, customer_id: str,
                         **kwargs) -> 'DetailedResponse':
        """
        Delete labeled data.

        Deletes all data associated with a specified customer ID. The method has no effect
        if no data is associated with the customer ID.
        You associate a customer ID with data by passing the `X-Watson-Metadata` header
        with a request that passes data. For more information about personal data and
        customer IDs, see [Information
        security](https://cloud.ibm.com/docs/visual-recognition?topic=visual-recognition-information-security).

        :param str customer_id: The customer ID for which all data is to be
               deleted.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if customer_id is None:
            raise ValueError('customer_id must be provided')

        headers = {}
        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V3',
                                      operation_id='delete_user_data')
        headers.update(sdk_headers)

        params = {'version': self.version, 'customer_id': customer_id}

        url = '/v3/user_data'
        request = self.prepare_request(method='DELETE',
                                       url=url,
                                       headers=headers,
                                       params=params)

        response = self.send(request)
        return response


class ClassifyEnums(object):

    class AcceptLanguage(Enum):
        """
        The desired language of parts of the response. See the response for details.
        """
        EN = 'en'
        AR = 'ar'
        DE = 'de'
        ES = 'es'
        FR = 'fr'
        IT = 'it'
        JA = 'ja'
        KO = 'ko'
        PT_BR = 'pt-br'
        ZH_CN = 'zh-cn'
        ZH_TW = 'zh-tw'


##############################################################################
# Models
##############################################################################


class Class():
    """
    A category within a classifier.

    :attr str class_: The name of the class.
    """

    def __init__(self, class_: str) -> None:
        """
        Initialize a Class object.

        :param str class_: The name of the class.
        """
        self.class_ = class_

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'Class':
        """Initialize a Class object from a json dictionary."""
        args = {}
        valid_keys = ['class_', 'class']
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class Class: ' +
                ', '.join(bad_keys))
        if 'class' in _dict:
            args['class_'] = _dict.get('class')
        else:
            raise ValueError(
                'Required property \'class\' not present in Class JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a Class object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'class_') and self.class_ is not None:
            _dict['class'] = self.class_
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this Class object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'Class') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'Class') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ClassResult():
    """
    Result of a class within a classifier.

    :attr str class_: Name of the class.
          Class names are translated in the language defined by the **Accept-Language**
          request header for the build-in classifier IDs (`default`, `food`, and
          `explicit`). Class names of custom classifiers are not translated. The response
          might not be in the specified language when the requested language is not
          supported or when there is no translation for the class name.
    :attr float score: Confidence score for the property in the range of 0 to 1. A
          higher score indicates greater likelihood that the class is depicted in the
          image. The default threshold for returning scores from a classifier is 0.5.
    :attr str type_hierarchy: (optional) Knowledge graph of the property. For
          example, `/fruit/pome/apple/eating apple/Granny Smith`. Included only if
          identified.
    """

    def __init__(self,
                 class_: str,
                 score: float,
                 *,
                 type_hierarchy: str = None) -> None:
        """
        Initialize a ClassResult object.

        :param str class_: Name of the class.
               Class names are translated in the language defined by the
               **Accept-Language** request header for the build-in classifier IDs
               (`default`, `food`, and `explicit`). Class names of custom classifiers are
               not translated. The response might not be in the specified language when
               the requested language is not supported or when there is no translation for
               the class name.
        :param float score: Confidence score for the property in the range of 0 to
               1. A higher score indicates greater likelihood that the class is depicted
               in the image. The default threshold for returning scores from a classifier
               is 0.5.
        :param str type_hierarchy: (optional) Knowledge graph of the property. For
               example, `/fruit/pome/apple/eating apple/Granny Smith`. Included only if
               identified.
        """
        self.class_ = class_
        self.score = score
        self.type_hierarchy = type_hierarchy

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ClassResult':
        """Initialize a ClassResult object from a json dictionary."""
        args = {}
        valid_keys = ['class_', 'class', 'score', 'type_hierarchy']
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class ClassResult: '
                + ', '.join(bad_keys))
        if 'class' in _dict:
            args['class_'] = _dict.get('class')
        else:
            raise ValueError(
                'Required property \'class\' not present in ClassResult JSON')
        if 'score' in _dict:
            args['score'] = _dict.get('score')
        else:
            raise ValueError(
                'Required property \'score\' not present in ClassResult JSON')
        if 'type_hierarchy' in _dict:
            args['type_hierarchy'] = _dict.get('type_hierarchy')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ClassResult object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'class_') and self.class_ is not None:
            _dict['class'] = self.class_
        if hasattr(self, 'score') and self.score is not None:
            _dict['score'] = self.score
        if hasattr(self, 'type_hierarchy') and self.type_hierarchy is not None:
            _dict['type_hierarchy'] = self.type_hierarchy
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ClassResult object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'ClassResult') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ClassResult') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ClassifiedImage():
    """
    Results for one image.

    :attr str source_url: (optional) Source of the image before any redirects. Not
          returned when the image is uploaded.
    :attr str resolved_url: (optional) Fully resolved URL of the image after
          redirects are followed. Not returned when the image is uploaded.
    :attr str image: (optional) Relative path of the image file if uploaded
          directly. Not returned when the image is passed by URL.
    :attr ErrorInfo error: (optional) Information about what might have caused a
          failure, such as an image that is too large. Not returned when there is no
          error.
    :attr List[ClassifierResult] classifiers: The classifiers.
    """

    def __init__(self,
                 classifiers: List['ClassifierResult'],
                 *,
                 source_url: str = None,
                 resolved_url: str = None,
                 image: str = None,
                 error: 'ErrorInfo' = None) -> None:
        """
        Initialize a ClassifiedImage object.

        :param List[ClassifierResult] classifiers: The classifiers.
        :param str source_url: (optional) Source of the image before any redirects.
               Not returned when the image is uploaded.
        :param str resolved_url: (optional) Fully resolved URL of the image after
               redirects are followed. Not returned when the image is uploaded.
        :param str image: (optional) Relative path of the image file if uploaded
               directly. Not returned when the image is passed by URL.
        :param ErrorInfo error: (optional) Information about what might have caused
               a failure, such as an image that is too large. Not returned when there is
               no error.
        """
        self.source_url = source_url
        self.resolved_url = resolved_url
        self.image = image
        self.error = error
        self.classifiers = classifiers

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ClassifiedImage':
        """Initialize a ClassifiedImage object from a json dictionary."""
        args = {}
        valid_keys = [
            'source_url', 'resolved_url', 'image', 'error', 'classifiers'
        ]
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class ClassifiedImage: '
                + ', '.join(bad_keys))
        if 'source_url' in _dict:
            args['source_url'] = _dict.get('source_url')
        if 'resolved_url' in _dict:
            args['resolved_url'] = _dict.get('resolved_url')
        if 'image' in _dict:
            args['image'] = _dict.get('image')
        if 'error' in _dict:
            args['error'] = ErrorInfo._from_dict(_dict.get('error'))
        if 'classifiers' in _dict:
            args['classifiers'] = [
                ClassifierResult._from_dict(x)
                for x in (_dict.get('classifiers'))
            ]
        else:
            raise ValueError(
                'Required property \'classifiers\' not present in ClassifiedImage JSON'
            )
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ClassifiedImage object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'source_url') and self.source_url is not None:
            _dict['source_url'] = self.source_url
        if hasattr(self, 'resolved_url') and self.resolved_url is not None:
            _dict['resolved_url'] = self.resolved_url
        if hasattr(self, 'image') and self.image is not None:
            _dict['image'] = self.image
        if hasattr(self, 'error') and self.error is not None:
            _dict['error'] = self.error._to_dict()
        if hasattr(self, 'classifiers') and self.classifiers is not None:
            _dict['classifiers'] = [x._to_dict() for x in self.classifiers]
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ClassifiedImage object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'ClassifiedImage') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ClassifiedImage') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ClassifiedImages():
    """
    Results for all images.

    :attr int custom_classes: (optional) Number of custom classes identified in the
          images.
    :attr int images_processed: (optional) Number of images processed for the API
          call.
    :attr List[ClassifiedImage] images: Classified images.
    :attr List[WarningInfo] warnings: (optional) Information about what might cause
          less than optimal output. For example, a request sent with a corrupt .zip file
          and a list of image URLs will still complete, but does not return the expected
          output. Not returned when there is no warning.
    """

    def __init__(self,
                 images: List['ClassifiedImage'],
                 *,
                 custom_classes: int = None,
                 images_processed: int = None,
                 warnings: List['WarningInfo'] = None) -> None:
        """
        Initialize a ClassifiedImages object.

        :param List[ClassifiedImage] images: Classified images.
        :param int custom_classes: (optional) Number of custom classes identified
               in the images.
        :param int images_processed: (optional) Number of images processed for the
               API call.
        :param List[WarningInfo] warnings: (optional) Information about what might
               cause less than optimal output. For example, a request sent with a corrupt
               .zip file and a list of image URLs will still complete, but does not return
               the expected output. Not returned when there is no warning.
        """
        self.custom_classes = custom_classes
        self.images_processed = images_processed
        self.images = images
        self.warnings = warnings

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ClassifiedImages':
        """Initialize a ClassifiedImages object from a json dictionary."""
        args = {}
        valid_keys = [
            'custom_classes', 'images_processed', 'images', 'warnings'
        ]
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class ClassifiedImages: '
                + ', '.join(bad_keys))
        if 'custom_classes' in _dict:
            args['custom_classes'] = _dict.get('custom_classes')
        if 'images_processed' in _dict:
            args['images_processed'] = _dict.get('images_processed')
        if 'images' in _dict:
            args['images'] = [
                ClassifiedImage._from_dict(x) for x in (_dict.get('images'))
            ]
        else:
            raise ValueError(
                'Required property \'images\' not present in ClassifiedImages JSON'
            )
        if 'warnings' in _dict:
            args['warnings'] = [
                WarningInfo._from_dict(x) for x in (_dict.get('warnings'))
            ]
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ClassifiedImages object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'custom_classes') and self.custom_classes is not None:
            _dict['custom_classes'] = self.custom_classes
        if hasattr(self,
                   'images_processed') and self.images_processed is not None:
            _dict['images_processed'] = self.images_processed
        if hasattr(self, 'images') and self.images is not None:
            _dict['images'] = [x._to_dict() for x in self.images]
        if hasattr(self, 'warnings') and self.warnings is not None:
            _dict['warnings'] = [x._to_dict() for x in self.warnings]
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ClassifiedImages object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'ClassifiedImages') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ClassifiedImages') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class Classifier():
    """
    Information about a classifier.

    :attr str classifier_id: ID of a classifier identified in the image.
    :attr str name: Name of the classifier.
    :attr str owner: (optional) Unique ID of the account who owns the classifier.
          Might not be returned by some requests.
    :attr str status: (optional) Training status of classifier.
    :attr bool core_ml_enabled: (optional) Whether the classifier can be downloaded
          as a Core ML model after the training status is `ready`.
    :attr str explanation: (optional) If classifier training has failed, this field
          might explain why.
    :attr datetime created: (optional) Date and time in Coordinated Universal Time
          (UTC) that the classifier was created.
    :attr List[Class] classes: (optional) Classes that define a classifier.
    :attr datetime retrained: (optional) Date and time in Coordinated Universal Time
          (UTC) that the classifier was updated. Might not be returned by some requests.
          Identical to `updated` and retained for backward compatibility.
    :attr datetime updated: (optional) Date and time in Coordinated Universal Time
          (UTC) that the classifier was most recently updated. The field matches either
          `retrained` or `created`. Might not be returned by some requests.
    """

    def __init__(self,
                 classifier_id: str,
                 name: str,
                 *,
                 owner: str = None,
                 status: str = None,
                 core_ml_enabled: bool = None,
                 explanation: str = None,
                 created: datetime = None,
                 classes: List['Class'] = None,
                 retrained: datetime = None,
                 updated: datetime = None) -> None:
        """
        Initialize a Classifier object.

        :param str classifier_id: ID of a classifier identified in the image.
        :param str name: Name of the classifier.
        :param str owner: (optional) Unique ID of the account who owns the
               classifier. Might not be returned by some requests.
        :param str status: (optional) Training status of classifier.
        :param bool core_ml_enabled: (optional) Whether the classifier can be
               downloaded as a Core ML model after the training status is `ready`.
        :param str explanation: (optional) If classifier training has failed, this
               field might explain why.
        :param datetime created: (optional) Date and time in Coordinated Universal
               Time (UTC) that the classifier was created.
        :param List[Class] classes: (optional) Classes that define a classifier.
        :param datetime retrained: (optional) Date and time in Coordinated
               Universal Time (UTC) that the classifier was updated. Might not be returned
               by some requests. Identical to `updated` and retained for backward
               compatibility.
        :param datetime updated: (optional) Date and time in Coordinated Universal
               Time (UTC) that the classifier was most recently updated. The field matches
               either `retrained` or `created`. Might not be returned by some requests.
        """
        self.classifier_id = classifier_id
        self.name = name
        self.owner = owner
        self.status = status
        self.core_ml_enabled = core_ml_enabled
        self.explanation = explanation
        self.created = created
        self.classes = classes
        self.retrained = retrained
        self.updated = updated

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'Classifier':
        """Initialize a Classifier object from a json dictionary."""
        args = {}
        valid_keys = [
            'classifier_id', 'name', 'owner', 'status', 'core_ml_enabled',
            'explanation', 'created', 'classes', 'retrained', 'updated'
        ]
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class Classifier: '
                + ', '.join(bad_keys))
        if 'classifier_id' in _dict:
            args['classifier_id'] = _dict.get('classifier_id')
        else:
            raise ValueError(
                'Required property \'classifier_id\' not present in Classifier JSON'
            )
        if 'name' in _dict:
            args['name'] = _dict.get('name')
        else:
            raise ValueError(
                'Required property \'name\' not present in Classifier JSON')
        if 'owner' in _dict:
            args['owner'] = _dict.get('owner')
        if 'status' in _dict:
            args['status'] = _dict.get('status')
        if 'core_ml_enabled' in _dict:
            args['core_ml_enabled'] = _dict.get('core_ml_enabled')
        if 'explanation' in _dict:
            args['explanation'] = _dict.get('explanation')
        if 'created' in _dict:
            args['created'] = string_to_datetime(_dict.get('created'))
        if 'classes' in _dict:
            args['classes'] = [
                Class._from_dict(x) for x in (_dict.get('classes'))
            ]
        if 'retrained' in _dict:
            args['retrained'] = string_to_datetime(_dict.get('retrained'))
        if 'updated' in _dict:
            args['updated'] = string_to_datetime(_dict.get('updated'))
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a Classifier object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'classifier_id') and self.classifier_id is not None:
            _dict['classifier_id'] = self.classifier_id
        if hasattr(self, 'name') and self.name is not None:
            _dict['name'] = self.name
        if hasattr(self, 'owner') and self.owner is not None:
            _dict['owner'] = self.owner
        if hasattr(self, 'status') and self.status is not None:
            _dict['status'] = self.status
        if hasattr(self,
                   'core_ml_enabled') and self.core_ml_enabled is not None:
            _dict['core_ml_enabled'] = self.core_ml_enabled
        if hasattr(self, 'explanation') and self.explanation is not None:
            _dict['explanation'] = self.explanation
        if hasattr(self, 'created') and self.created is not None:
            _dict['created'] = datetime_to_string(self.created)
        if hasattr(self, 'classes') and self.classes is not None:
            _dict['classes'] = [x._to_dict() for x in self.classes]
        if hasattr(self, 'retrained') and self.retrained is not None:
            _dict['retrained'] = datetime_to_string(self.retrained)
        if hasattr(self, 'updated') and self.updated is not None:
            _dict['updated'] = datetime_to_string(self.updated)
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this Classifier object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'Classifier') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'Classifier') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

    class StatusEnum(Enum):
        """
        Training status of classifier.
        """
        READY = "ready"
        TRAINING = "training"
        RETRAINING = "retraining"
        FAILED = "failed"


class ClassifierResult():
    """
    Classifier and score combination.

    :attr str name: Name of the classifier.
    :attr str classifier_id: ID of a classifier identified in the image.
    :attr List[ClassResult] classes: Classes within the classifier.
    """

    def __init__(self, name: str, classifier_id: str,
                 classes: List['ClassResult']) -> None:
        """
        Initialize a ClassifierResult object.

        :param str name: Name of the classifier.
        :param str classifier_id: ID of a classifier identified in the image.
        :param List[ClassResult] classes: Classes within the classifier.
        """
        self.name = name
        self.classifier_id = classifier_id
        self.classes = classes

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ClassifierResult':
        """Initialize a ClassifierResult object from a json dictionary."""
        args = {}
        valid_keys = ['name', 'classifier_id', 'classes']
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class ClassifierResult: '
                + ', '.join(bad_keys))
        if 'name' in _dict:
            args['name'] = _dict.get('name')
        else:
            raise ValueError(
                'Required property \'name\' not present in ClassifierResult JSON'
            )
        if 'classifier_id' in _dict:
            args['classifier_id'] = _dict.get('classifier_id')
        else:
            raise ValueError(
                'Required property \'classifier_id\' not present in ClassifierResult JSON'
            )
        if 'classes' in _dict:
            args['classes'] = [
                ClassResult._from_dict(x) for x in (_dict.get('classes'))
            ]
        else:
            raise ValueError(
                'Required property \'classes\' not present in ClassifierResult JSON'
            )
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ClassifierResult object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'name') and self.name is not None:
            _dict['name'] = self.name
        if hasattr(self, 'classifier_id') and self.classifier_id is not None:
            _dict['classifier_id'] = self.classifier_id
        if hasattr(self, 'classes') and self.classes is not None:
            _dict['classes'] = [x._to_dict() for x in self.classes]
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ClassifierResult object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'ClassifierResult') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ClassifierResult') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class Classifiers():
    """
    A container for the list of classifiers.

    :attr List[Classifier] classifiers: List of classifiers.
    """

    def __init__(self, classifiers: List['Classifier']) -> None:
        """
        Initialize a Classifiers object.

        :param List[Classifier] classifiers: List of classifiers.
        """
        self.classifiers = classifiers

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'Classifiers':
        """Initialize a Classifiers object from a json dictionary."""
        args = {}
        valid_keys = ['classifiers']
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class Classifiers: '
                + ', '.join(bad_keys))
        if 'classifiers' in _dict:
            args['classifiers'] = [
                Classifier._from_dict(x) for x in (_dict.get('classifiers'))
            ]
        else:
            raise ValueError(
                'Required property \'classifiers\' not present in Classifiers JSON'
            )
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a Classifiers object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'classifiers') and self.classifiers is not None:
            _dict['classifiers'] = [x._to_dict() for x in self.classifiers]
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this Classifiers object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'Classifiers') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'Classifiers') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ErrorInfo():
    """
    Information about what might have caused a failure, such as an image that is too
    large. Not returned when there is no error.

    :attr int code: HTTP status code.
    :attr str description: Human-readable error description. For example, `File size
          limit exceeded`.
    :attr str error_id: Codified error string. For example, `limit_exceeded`.
    """

    def __init__(self, code: int, description: str, error_id: str) -> None:
        """
        Initialize a ErrorInfo object.

        :param int code: HTTP status code.
        :param str description: Human-readable error description. For example,
               `File size limit exceeded`.
        :param str error_id: Codified error string. For example, `limit_exceeded`.
        """
        self.code = code
        self.description = description
        self.error_id = error_id

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ErrorInfo':
        """Initialize a ErrorInfo object from a json dictionary."""
        args = {}
        valid_keys = ['code', 'description', 'error_id']
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class ErrorInfo: '
                + ', '.join(bad_keys))
        if 'code' in _dict:
            args['code'] = _dict.get('code')
        else:
            raise ValueError(
                'Required property \'code\' not present in ErrorInfo JSON')
        if 'description' in _dict:
            args['description'] = _dict.get('description')
        else:
            raise ValueError(
                'Required property \'description\' not present in ErrorInfo JSON'
            )
        if 'error_id' in _dict:
            args['error_id'] = _dict.get('error_id')
        else:
            raise ValueError(
                'Required property \'error_id\' not present in ErrorInfo JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ErrorInfo object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'code') and self.code is not None:
            _dict['code'] = self.code
        if hasattr(self, 'description') and self.description is not None:
            _dict['description'] = self.description
        if hasattr(self, 'error_id') and self.error_id is not None:
            _dict['error_id'] = self.error_id
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ErrorInfo object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'ErrorInfo') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ErrorInfo') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class WarningInfo():
    """
    Information about something that went wrong.

    :attr str warning_id: Codified warning string, such as `limit_reached`.
    :attr str description: Information about the error.
    """

    def __init__(self, warning_id: str, description: str) -> None:
        """
        Initialize a WarningInfo object.

        :param str warning_id: Codified warning string, such as `limit_reached`.
        :param str description: Information about the error.
        """
        self.warning_id = warning_id
        self.description = description

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'WarningInfo':
        """Initialize a WarningInfo object from a json dictionary."""
        args = {}
        valid_keys = ['warning_id', 'description']
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class WarningInfo: '
                + ', '.join(bad_keys))
        if 'warning_id' in _dict:
            args['warning_id'] = _dict.get('warning_id')
        else:
            raise ValueError(
                'Required property \'warning_id\' not present in WarningInfo JSON'
            )
        if 'description' in _dict:
            args['description'] = _dict.get('description')
        else:
            raise ValueError(
                'Required property \'description\' not present in WarningInfo JSON'
            )
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a WarningInfo object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'warning_id') and self.warning_id is not None:
            _dict['warning_id'] = self.warning_id
        if hasattr(self, 'description') and self.description is not None:
            _dict['description'] = self.description
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this WarningInfo object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'WarningInfo') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'WarningInfo') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other



# coding: utf-8

# (C) Copyright IBM Corp. 2016, 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
The IBM Watson&trade; Tone Analyzer service uses linguistic analysis to detect emotional
and language tones in written text. The service can analyze tone at both the document and
sentence levels. You can use the service to understand how your written communications are
perceived and then to improve the tone of your communications. Businesses can use the
service to learn the tone of their customers' communications and to respond to each
customer appropriately, or to understand and improve their customer conversations.
**Note:** Request logging is disabled for the Tone Analyzer service. Regardless of whether
you set the `X-Watson-Learning-Opt-Out` request header, the service does not log or retain
data from requests and responses.
"""

import json
from ibm_cloud_sdk_core.authenticators.authenticator import Authenticator
from .common import get_sdk_headers
from enum import Enum
from ibm_cloud_sdk_core import BaseService
from ibm_cloud_sdk_core import DetailedResponse
from ibm_cloud_sdk_core.get_authenticator import get_authenticator_from_environment
from typing import Dict
from typing import List

##############################################################################
# Service
##############################################################################


class ToneAnalyzerV3(BaseService):
    """The Tone Analyzer V3 service."""

    DEFAULT_SERVICE_URL = 'https://api.us-south.tone-analyzer.watson.cloud.ibm.com'
    DEFAULT_SERVICE_NAME = 'tone_analyzer'

    def __init__(
        self,
        version: str,
        authenticator: Authenticator = None,
        service_name: str = DEFAULT_SERVICE_NAME,
    ) -> None:
        """
        Construct a new client for the Tone Analyzer service.

        :param str version: The API version date to use with the service, in
               "YYYY-MM-DD" format. Whenever the API is changed in a backwards
               incompatible way, a new minor version of the API is released.
               The service uses the API version for the date you specify, or
               the most recent version before that date. Note that you should
               not programmatically specify the current date at runtime, in
               case the API has been updated since your application's release.
               Instead, specify a version date that is compatible with your
               application, and don't change it until your application is
               ready for a later version.

        :param Authenticator authenticator: The authenticator specifies the authentication mechanism.
               Get up to date information from https://github.com/IBM/python-sdk-core/blob/master/README.md
               about initializing the authenticator of your choice.
        """
        if not authenticator:
            authenticator = get_authenticator_from_environment(service_name)
        BaseService.__init__(self,
                             service_url=self.DEFAULT_SERVICE_URL,
                             authenticator=authenticator,
                             disable_ssl_verification=False)
        self.version = version
        self.configure_service(service_name)

    #########################
    # Methods
    #########################

    def tone(self,
             tone_input: object,
             *,
             content_type: str = None,
             sentences: bool = None,
             tones: List[str] = None,
             content_language: str = None,
             accept_language: str = None,
             **kwargs) -> 'DetailedResponse':
        """
        Analyze general tone.

        Use the general-purpose endpoint to analyze the tone of your input content. The
        service analyzes the content for emotional and language tones. The method always
        analyzes the tone of the full document; by default, it also analyzes the tone of
        each individual sentence of the content.
        You can submit no more than 128 KB of total input content and no more than 1000
        individual sentences in JSON, plain text, or HTML format. The service analyzes the
        first 1000 sentences for document-level analysis and only the first 100 sentences
        for sentence-level analysis.
        Per the JSON specification, the default character encoding for JSON content is
        effectively always UTF-8; per the HTTP specification, the default encoding for
        plain text and HTML is ISO-8859-1 (effectively, the ASCII character set). When
        specifying a content type of plain text or HTML, include the `charset` parameter
        to indicate the character encoding of the input text; for example: `Content-Type:
        text/plain;charset=utf-8`. For `text/html`, the service removes HTML tags and
        analyzes only the textual content.
        **See also:** [Using the general-purpose
        endpoint](https://cloud.ibm.com/docs/tone-analyzer?topic=tone-analyzer-utgpe#utgpe).

        :param ToneInput tone_input: JSON, plain text, or HTML input that contains
               the content to be analyzed. For JSON input, provide an object of type
               `ToneInput`.
        :param str content_type: (optional) The type of the input. A character
               encoding can be specified by including a `charset` parameter. For example,
               'text/plain;charset=utf-8'.
        :param bool sentences: (optional) Indicates whether the service is to
               return an analysis of each individual sentence in addition to its analysis
               of the full document. If `true` (the default), the service returns results
               for each sentence.
        :param List[str] tones: (optional) **`2017-09-21`:** Deprecated. The
               service continues to accept the parameter for backward-compatibility, but
               the parameter no longer affects the response.
               **`2016-05-19`:** A comma-separated list of tones for which the service is
               to return its analysis of the input; the indicated tones apply both to the
               full document and to individual sentences of the document. You can specify
               one or more of the valid values. Omit the parameter to request results for
               all three tones.
        :param str content_language: (optional) The language of the input text for
               the request: English or French. Regional variants are treated as their
               parent language; for example, `en-US` is interpreted as `en`. The input
               content must match the specified language. Do not submit content that
               contains both languages. You can use different languages for
               **Content-Language** and **Accept-Language**.
               * **`2017-09-21`:** Accepts `en` or `fr`.
               * **`2016-05-19`:** Accepts only `en`.
        :param str accept_language: (optional) The desired language of the
               response. For two-character arguments, regional variants are treated as
               their parent language; for example, `en-US` is interpreted as `en`. You can
               use different languages for **Content-Language** and **Accept-Language**.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if tone_input is None:
            raise ValueError('tone_input must be provided')
        if isinstance(tone_input, ToneInput):
            tone_input = self._convert_model(tone_input)

        headers = {
            'Content-Type': content_type,
            'Content-Language': content_language,
            'Accept-Language': accept_language
        }
        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V3',
                                      operation_id='tone')
        headers.update(sdk_headers)

        params = {
            'version': self.version,
            'sentences': sentences,
            'tones': self._convert_list(tones)
        }

        if content_type == 'application/json' and isinstance(tone_input, dict):
            data = json.dumps(tone_input)
        else:
            data = tone_input

        url = '/v3/tone'
        request = self.prepare_request(method='POST',
                                       url=url,
                                       headers=headers,
                                       params=params,
                                       data=data)

        response = self.send(request)
        return response

    def tone_chat(self,
                  utterances: List['Utterance'],
                  *,
                  content_language: str = None,
                  accept_language: str = None,
                  **kwargs) -> 'DetailedResponse':
        """
        Analyze customer-engagement tone.

        Use the customer-engagement endpoint to analyze the tone of customer service and
        customer support conversations. For each utterance of a conversation, the method
        reports the most prevalent subset of the following seven tones: sad, frustrated,
        satisfied, excited, polite, impolite, and sympathetic.
        If you submit more than 50 utterances, the service returns a warning for the
        overall content and analyzes only the first 50 utterances. If you submit a single
        utterance that contains more than 500 characters, the service returns an error for
        that utterance and does not analyze the utterance. The request fails if all
        utterances have more than 500 characters. Per the JSON specification, the default
        character encoding for JSON content is effectively always UTF-8.
        **See also:** [Using the customer-engagement
        endpoint](https://cloud.ibm.com/docs/tone-analyzer?topic=tone-analyzer-utco#utco).

        :param List[Utterance] utterances: An array of `Utterance` objects that
               provides the input content that the service is to analyze.
        :param str content_language: (optional) The language of the input text for
               the request: English or French. Regional variants are treated as their
               parent language; for example, `en-US` is interpreted as `en`. The input
               content must match the specified language. Do not submit content that
               contains both languages. You can use different languages for
               **Content-Language** and **Accept-Language**.
               * **`2017-09-21`:** Accepts `en` or `fr`.
               * **`2016-05-19`:** Accepts only `en`.
        :param str accept_language: (optional) The desired language of the
               response. For two-character arguments, regional variants are treated as
               their parent language; for example, `en-US` is interpreted as `en`. You can
               use different languages for **Content-Language** and **Accept-Language**.
        :param dict headers: A `dict` containing the request headers
        :return: A `DetailedResponse` containing the result, headers and HTTP status code.
        :rtype: DetailedResponse
        """

        if utterances is None:
            raise ValueError('utterances must be provided')
        utterances = [self._convert_model(x) for x in utterances]

        headers = {
            'Content-Language': content_language,
            'Accept-Language': accept_language
        }
        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        sdk_headers = get_sdk_headers(service_name=self.DEFAULT_SERVICE_NAME,
                                      service_version='V3',
                                      operation_id='tone_chat')
        headers.update(sdk_headers)

        params = {'version': self.version}

        data = {'utterances': utterances}

        url = '/v3/tone_chat'
        request = self.prepare_request(method='POST',
                                       url=url,
                                       headers=headers,
                                       params=params,
                                       data=data)

        response = self.send(request)
        return response


class ToneEnums(object):

    class ContentType(Enum):
        """
        The type of the input. A character encoding can be specified by including a
        `charset` parameter. For example, 'text/plain;charset=utf-8'.
        """
        APPLICATION_JSON = 'application/json'
        TEXT_PLAIN = 'text/plain'
        TEXT_HTML = 'text/html'

    class Tones(Enum):
        """
        **`2017-09-21`:** Deprecated. The service continues to accept the parameter for
        backward-compatibility, but the parameter no longer affects the response.
        **`2016-05-19`:** A comma-separated list of tones for which the service is to
        return its analysis of the input; the indicated tones apply both to the full
        document and to individual sentences of the document. You can specify one or more
        of the valid values. Omit the parameter to request results for all three tones.
        """
        EMOTION = 'emotion'
        LANGUAGE = 'language'
        SOCIAL = 'social'

    class ContentLanguage(Enum):
        """
        The language of the input text for the request: English or French. Regional
        variants are treated as their parent language; for example, `en-US` is interpreted
        as `en`. The input content must match the specified language. Do not submit
        content that contains both languages. You can use different languages for
        **Content-Language** and **Accept-Language**.
        * **`2017-09-21`:** Accepts `en` or `fr`.
        * **`2016-05-19`:** Accepts only `en`.
        """
        EN = 'en'
        FR = 'fr'

    class AcceptLanguage(Enum):
        """
        The desired language of the response. For two-character arguments, regional
        variants are treated as their parent language; for example, `en-US` is interpreted
        as `en`. You can use different languages for **Content-Language** and
        **Accept-Language**.
        """
        AR = 'ar'
        DE = 'de'
        EN = 'en'
        ES = 'es'
        FR = 'fr'
        IT = 'it'
        JA = 'ja'
        KO = 'ko'
        PT_BR = 'pt-br'
        ZH_CN = 'zh-cn'
        ZH_TW = 'zh-tw'


class ToneChatEnums(object):

    class ContentLanguage(Enum):
        """
        The language of the input text for the request: English or French. Regional
        variants are treated as their parent language; for example, `en-US` is interpreted
        as `en`. The input content must match the specified language. Do not submit
        content that contains both languages. You can use different languages for
        **Content-Language** and **Accept-Language**.
        * **`2017-09-21`:** Accepts `en` or `fr`.
        * **`2016-05-19`:** Accepts only `en`.
        """
        EN = 'en'
        FR = 'fr'

    class AcceptLanguage(Enum):
        """
        The desired language of the response. For two-character arguments, regional
        variants are treated as their parent language; for example, `en-US` is interpreted
        as `en`. You can use different languages for **Content-Language** and
        **Accept-Language**.
        """
        AR = 'ar'
        DE = 'de'
        EN = 'en'
        ES = 'es'
        FR = 'fr'
        IT = 'it'
        JA = 'ja'
        KO = 'ko'
        PT_BR = 'pt-br'
        ZH_CN = 'zh-cn'
        ZH_TW = 'zh-tw'


##############################################################################
# Models
##############################################################################


class DocumentAnalysis():
    """
    The results of the analysis for the full input content.

    :attr List[ToneScore] tones: (optional) **`2017-09-21`:** An array of
          `ToneScore` objects that provides the results of the analysis for each
          qualifying tone of the document. The array includes results for any tone whose
          score is at least 0.5. The array is empty if no tone has a score that meets this
          threshold. **`2016-05-19`:** Not returned.
    :attr List[ToneCategory] tone_categories: (optional) **`2017-09-21`:** Not
          returned. **`2016-05-19`:** An array of `ToneCategory` objects that provides the
          results of the tone analysis for the full document of the input content. The
          service returns results only for the tones specified with the `tones` parameter
          of the request.
    :attr str warning: (optional) **`2017-09-21`:** A warning message if the overall
          content exceeds 128 KB or contains more than 1000 sentences. The service
          analyzes only the first 1000 sentences for document-level analysis and the first
          100 sentences for sentence-level analysis. **`2016-05-19`:** Not returned.
    """

    def __init__(self,
                 *,
                 tones: List['ToneScore'] = None,
                 tone_categories: List['ToneCategory'] = None,
                 warning: str = None) -> None:
        """
        Initialize a DocumentAnalysis object.

        :param List[ToneScore] tones: (optional) **`2017-09-21`:** An array of
               `ToneScore` objects that provides the results of the analysis for each
               qualifying tone of the document. The array includes results for any tone
               whose score is at least 0.5. The array is empty if no tone has a score that
               meets this threshold. **`2016-05-19`:** Not returned.
        :param List[ToneCategory] tone_categories: (optional) **`2017-09-21`:** Not
               returned. **`2016-05-19`:** An array of `ToneCategory` objects that
               provides the results of the tone analysis for the full document of the
               input content. The service returns results only for the tones specified
               with the `tones` parameter of the request.
        :param str warning: (optional) **`2017-09-21`:** A warning message if the
               overall content exceeds 128 KB or contains more than 1000 sentences. The
               service analyzes only the first 1000 sentences for document-level analysis
               and the first 100 sentences for sentence-level analysis. **`2016-05-19`:**
               Not returned.
        """
        self.tones = tones
        self.tone_categories = tone_categories
        self.warning = warning

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'DocumentAnalysis':
        """Initialize a DocumentAnalysis object from a json dictionary."""
        args = {}
        valid_keys = ['tones', 'tone_categories', 'warning']
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class DocumentAnalysis: '
                + ', '.join(bad_keys))
        if 'tones' in _dict:
            args['tones'] = [
                ToneScore._from_dict(x) for x in (_dict.get('tones'))
            ]
        if 'tone_categories' in _dict:
            args['tone_categories'] = [
                ToneCategory._from_dict(x)
                for x in (_dict.get('tone_categories'))
            ]
        if 'warning' in _dict:
            args['warning'] = _dict.get('warning')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a DocumentAnalysis object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'tones') and self.tones is not None:
            _dict['tones'] = [x._to_dict() for x in self.tones]
        if hasattr(self,
                   'tone_categories') and self.tone_categories is not None:
            _dict['tone_categories'] = [
                x._to_dict() for x in self.tone_categories
            ]
        if hasattr(self, 'warning') and self.warning is not None:
            _dict['warning'] = self.warning
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this DocumentAnalysis object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'DocumentAnalysis') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'DocumentAnalysis') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class SentenceAnalysis():
    """
    The results of the analysis for the individual sentences of the input content.

    :attr int sentence_id: The unique identifier of a sentence of the input content.
          The first sentence has ID 0, and the ID of each subsequent sentence is
          incremented by one.
    :attr str text: The text of the input sentence.
    :attr List[ToneScore] tones: (optional) **`2017-09-21`:** An array of
          `ToneScore` objects that provides the results of the analysis for each
          qualifying tone of the sentence. The array includes results for any tone whose
          score is at least 0.5. The array is empty if no tone has a score that meets this
          threshold. **`2016-05-19`:** Not returned.
    :attr List[ToneCategory] tone_categories: (optional) **`2017-09-21`:** Not
          returned. **`2016-05-19`:** An array of `ToneCategory` objects that provides the
          results of the tone analysis for the sentence. The service returns results only
          for the tones specified with the `tones` parameter of the request.
    :attr int input_from: (optional) **`2017-09-21`:** Not returned.
          **`2016-05-19`:** The offset of the first character of the sentence in the
          overall input content.
    :attr int input_to: (optional) **`2017-09-21`:** Not returned. **`2016-05-19`:**
          The offset of the last character of the sentence in the overall input content.
    """

    def __init__(self,
                 sentence_id: int,
                 text: str,
                 *,
                 tones: List['ToneScore'] = None,
                 tone_categories: List['ToneCategory'] = None,
                 input_from: int = None,
                 input_to: int = None) -> None:
        """
        Initialize a SentenceAnalysis object.

        :param int sentence_id: The unique identifier of a sentence of the input
               content. The first sentence has ID 0, and the ID of each subsequent
               sentence is incremented by one.
        :param str text: The text of the input sentence.
        :param List[ToneScore] tones: (optional) **`2017-09-21`:** An array of
               `ToneScore` objects that provides the results of the analysis for each
               qualifying tone of the sentence. The array includes results for any tone
               whose score is at least 0.5. The array is empty if no tone has a score that
               meets this threshold. **`2016-05-19`:** Not returned.
        :param List[ToneCategory] tone_categories: (optional) **`2017-09-21`:** Not
               returned. **`2016-05-19`:** An array of `ToneCategory` objects that
               provides the results of the tone analysis for the sentence. The service
               returns results only for the tones specified with the `tones` parameter of
               the request.
        :param int input_from: (optional) **`2017-09-21`:** Not returned.
               **`2016-05-19`:** The offset of the first character of the sentence in the
               overall input content.
        :param int input_to: (optional) **`2017-09-21`:** Not returned.
               **`2016-05-19`:** The offset of the last character of the sentence in the
               overall input content.
        """
        self.sentence_id = sentence_id
        self.text = text
        self.tones = tones
        self.tone_categories = tone_categories
        self.input_from = input_from
        self.input_to = input_to

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'SentenceAnalysis':
        """Initialize a SentenceAnalysis object from a json dictionary."""
        args = {}
        valid_keys = [
            'sentence_id', 'text', 'tones', 'tone_categories', 'input_from',
            'input_to'
        ]
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class SentenceAnalysis: '
                + ', '.join(bad_keys))
        if 'sentence_id' in _dict:
            args['sentence_id'] = _dict.get('sentence_id')
        else:
            raise ValueError(
                'Required property \'sentence_id\' not present in SentenceAnalysis JSON'
            )
        if 'text' in _dict:
            args['text'] = _dict.get('text')
        else:
            raise ValueError(
                'Required property \'text\' not present in SentenceAnalysis JSON'
            )
        if 'tones' in _dict:
            args['tones'] = [
                ToneScore._from_dict(x) for x in (_dict.get('tones'))
            ]
        if 'tone_categories' in _dict:
            args['tone_categories'] = [
                ToneCategory._from_dict(x)
                for x in (_dict.get('tone_categories'))
            ]
        if 'input_from' in _dict:
            args['input_from'] = _dict.get('input_from')
        if 'input_to' in _dict:
            args['input_to'] = _dict.get('input_to')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a SentenceAnalysis object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'sentence_id') and self.sentence_id is not None:
            _dict['sentence_id'] = self.sentence_id
        if hasattr(self, 'text') and self.text is not None:
            _dict['text'] = self.text
        if hasattr(self, 'tones') and self.tones is not None:
            _dict['tones'] = [x._to_dict() for x in self.tones]
        if hasattr(self,
                   'tone_categories') and self.tone_categories is not None:
            _dict['tone_categories'] = [
                x._to_dict() for x in self.tone_categories
            ]
        if hasattr(self, 'input_from') and self.input_from is not None:
            _dict['input_from'] = self.input_from
        if hasattr(self, 'input_to') and self.input_to is not None:
            _dict['input_to'] = self.input_to
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this SentenceAnalysis object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'SentenceAnalysis') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'SentenceAnalysis') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ToneAnalysis():
    """
    The tone analysis results for the input from the general-purpose endpoint.

    :attr DocumentAnalysis document_tone: The results of the analysis for the full
          input content.
    :attr List[SentenceAnalysis] sentences_tone: (optional) An array of
          `SentenceAnalysis` objects that provides the results of the analysis for the
          individual sentences of the input content. The service returns results only for
          the first 100 sentences of the input. The field is omitted if the `sentences`
          parameter of the request is set to `false`.
    """

    def __init__(self,
                 document_tone: 'DocumentAnalysis',
                 *,
                 sentences_tone: List['SentenceAnalysis'] = None) -> None:
        """
        Initialize a ToneAnalysis object.

        :param DocumentAnalysis document_tone: The results of the analysis for the
               full input content.
        :param List[SentenceAnalysis] sentences_tone: (optional) An array of
               `SentenceAnalysis` objects that provides the results of the analysis for
               the individual sentences of the input content. The service returns results
               only for the first 100 sentences of the input. The field is omitted if the
               `sentences` parameter of the request is set to `false`.
        """
        self.document_tone = document_tone
        self.sentences_tone = sentences_tone

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ToneAnalysis':
        """Initialize a ToneAnalysis object from a json dictionary."""
        args = {}
        valid_keys = ['document_tone', 'sentences_tone']
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class ToneAnalysis: '
                + ', '.join(bad_keys))
        if 'document_tone' in _dict:
            args['document_tone'] = DocumentAnalysis._from_dict(
                _dict.get('document_tone'))
        else:
            raise ValueError(
                'Required property \'document_tone\' not present in ToneAnalysis JSON'
            )
        if 'sentences_tone' in _dict:
            args['sentences_tone'] = [
                SentenceAnalysis._from_dict(x)
                for x in (_dict.get('sentences_tone'))
            ]
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ToneAnalysis object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'document_tone') and self.document_tone is not None:
            _dict['document_tone'] = self.document_tone._to_dict()
        if hasattr(self, 'sentences_tone') and self.sentences_tone is not None:
            _dict['sentences_tone'] = [
                x._to_dict() for x in self.sentences_tone
            ]
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ToneAnalysis object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'ToneAnalysis') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ToneAnalysis') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ToneCategory():
    """
    The category for a tone from the input content.

    :attr List[ToneScore] tones: An array of `ToneScore` objects that provides the
          results for the tones of the category.
    :attr str category_id: The unique, non-localized identifier of the category for
          the results. The service can return results for the following category IDs:
          `emotion_tone`, `language_tone`, and `social_tone`.
    :attr str category_name: The user-visible, localized name of the category.
    """

    def __init__(self, tones: List['ToneScore'], category_id: str,
                 category_name: str) -> None:
        """
        Initialize a ToneCategory object.

        :param List[ToneScore] tones: An array of `ToneScore` objects that provides
               the results for the tones of the category.
        :param str category_id: The unique, non-localized identifier of the
               category for the results. The service can return results for the following
               category IDs: `emotion_tone`, `language_tone`, and `social_tone`.
        :param str category_name: The user-visible, localized name of the category.
        """
        self.tones = tones
        self.category_id = category_id
        self.category_name = category_name

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ToneCategory':
        """Initialize a ToneCategory object from a json dictionary."""
        args = {}
        valid_keys = ['tones', 'category_id', 'category_name']
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class ToneCategory: '
                + ', '.join(bad_keys))
        if 'tones' in _dict:
            args['tones'] = [
                ToneScore._from_dict(x) for x in (_dict.get('tones'))
            ]
        else:
            raise ValueError(
                'Required property \'tones\' not present in ToneCategory JSON')
        if 'category_id' in _dict:
            args['category_id'] = _dict.get('category_id')
        else:
            raise ValueError(
                'Required property \'category_id\' not present in ToneCategory JSON'
            )
        if 'category_name' in _dict:
            args['category_name'] = _dict.get('category_name')
        else:
            raise ValueError(
                'Required property \'category_name\' not present in ToneCategory JSON'
            )
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ToneCategory object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'tones') and self.tones is not None:
            _dict['tones'] = [x._to_dict() for x in self.tones]
        if hasattr(self, 'category_id') and self.category_id is not None:
            _dict['category_id'] = self.category_id
        if hasattr(self, 'category_name') and self.category_name is not None:
            _dict['category_name'] = self.category_name
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ToneCategory object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'ToneCategory') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ToneCategory') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ToneChatScore():
    """
    The score for an utterance from the input content.

    :attr float score: The score for the tone in the range of 0.5 to 1. A score
          greater than 0.75 indicates a high likelihood that the tone is perceived in the
          utterance.
    :attr str tone_id: The unique, non-localized identifier of the tone for the
          results. The service returns results only for tones whose scores meet a minimum
          threshold of 0.5.
    :attr str tone_name: The user-visible, localized name of the tone.
    """

    def __init__(self, score: float, tone_id: str, tone_name: str) -> None:
        """
        Initialize a ToneChatScore object.

        :param float score: The score for the tone in the range of 0.5 to 1. A
               score greater than 0.75 indicates a high likelihood that the tone is
               perceived in the utterance.
        :param str tone_id: The unique, non-localized identifier of the tone for
               the results. The service returns results only for tones whose scores meet a
               minimum threshold of 0.5.
        :param str tone_name: The user-visible, localized name of the tone.
        """
        self.score = score
        self.tone_id = tone_id
        self.tone_name = tone_name

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ToneChatScore':
        """Initialize a ToneChatScore object from a json dictionary."""
        args = {}
        valid_keys = ['score', 'tone_id', 'tone_name']
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class ToneChatScore: '
                + ', '.join(bad_keys))
        if 'score' in _dict:
            args['score'] = _dict.get('score')
        else:
            raise ValueError(
                'Required property \'score\' not present in ToneChatScore JSON')
        if 'tone_id' in _dict:
            args['tone_id'] = _dict.get('tone_id')
        else:
            raise ValueError(
                'Required property \'tone_id\' not present in ToneChatScore JSON'
            )
        if 'tone_name' in _dict:
            args['tone_name'] = _dict.get('tone_name')
        else:
            raise ValueError(
                'Required property \'tone_name\' not present in ToneChatScore JSON'
            )
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ToneChatScore object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'score') and self.score is not None:
            _dict['score'] = self.score
        if hasattr(self, 'tone_id') and self.tone_id is not None:
            _dict['tone_id'] = self.tone_id
        if hasattr(self, 'tone_name') and self.tone_name is not None:
            _dict['tone_name'] = self.tone_name
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ToneChatScore object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'ToneChatScore') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ToneChatScore') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other

    class ToneIdEnum(Enum):
        """
        The unique, non-localized identifier of the tone for the results. The service
        returns results only for tones whose scores meet a minimum threshold of 0.5.
        """
        EXCITED = "excited"
        FRUSTRATED = "frustrated"
        IMPOLITE = "impolite"
        POLITE = "polite"
        SAD = "sad"
        SATISFIED = "satisfied"
        SYMPATHETIC = "sympathetic"


class ToneInput():
    """
    Input for the general-purpose endpoint.

    :attr str text: The input content that the service is to analyze.
    """

    def __init__(self, text: str) -> None:
        """
        Initialize a ToneInput object.

        :param str text: The input content that the service is to analyze.
        """
        self.text = text

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ToneInput':
        """Initialize a ToneInput object from a json dictionary."""
        args = {}
        valid_keys = ['text']
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class ToneInput: '
                + ', '.join(bad_keys))
        if 'text' in _dict:
            args['text'] = _dict.get('text')
        else:
            raise ValueError(
                'Required property \'text\' not present in ToneInput JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ToneInput object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'text') and self.text is not None:
            _dict['text'] = self.text
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ToneInput object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'ToneInput') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ToneInput') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class ToneScore():
    """
    The score for a tone from the input content.

    :attr float score: The score for the tone.
          * **`2017-09-21`:** The score that is returned lies in the range of 0.5 to 1. A
          score greater than 0.75 indicates a high likelihood that the tone is perceived
          in the content.
          * **`2016-05-19`:** The score that is returned lies in the range of 0 to 1. A
          score less than 0.5 indicates that the tone is unlikely to be perceived in the
          content; a score greater than 0.75 indicates a high likelihood that the tone is
          perceived.
    :attr str tone_id: The unique, non-localized identifier of the tone.
          * **`2017-09-21`:** The service can return results for the following tone IDs:
          `anger`, `fear`, `joy`, and `sadness` (emotional tones); `analytical`,
          `confident`, and `tentative` (language tones). The service returns results only
          for tones whose scores meet a minimum threshold of 0.5.
          * **`2016-05-19`:** The service can return results for the following tone IDs of
          the different categories: for the `emotion` category: `anger`, `disgust`,
          `fear`, `joy`, and `sadness`; for the `language` category: `analytical`,
          `confident`, and `tentative`; for the `social` category: `openness_big5`,
          `conscientiousness_big5`, `extraversion_big5`, `agreeableness_big5`, and
          `emotional_range_big5`. The service returns scores for all tones of a category,
          regardless of their values.
    :attr str tone_name: The user-visible, localized name of the tone.
    """

    def __init__(self, score: float, tone_id: str, tone_name: str) -> None:
        """
        Initialize a ToneScore object.

        :param float score: The score for the tone.
               * **`2017-09-21`:** The score that is returned lies in the range of 0.5 to
               1. A score greater than 0.75 indicates a high likelihood that the tone is
               perceived in the content.
               * **`2016-05-19`:** The score that is returned lies in the range of 0 to 1.
               A score less than 0.5 indicates that the tone is unlikely to be perceived
               in the content; a score greater than 0.75 indicates a high likelihood that
               the tone is perceived.
        :param str tone_id: The unique, non-localized identifier of the tone.
               * **`2017-09-21`:** The service can return results for the following tone
               IDs: `anger`, `fear`, `joy`, and `sadness` (emotional tones); `analytical`,
               `confident`, and `tentative` (language tones). The service returns results
               only for tones whose scores meet a minimum threshold of 0.5.
               * **`2016-05-19`:** The service can return results for the following tone
               IDs of the different categories: for the `emotion` category: `anger`,
               `disgust`, `fear`, `joy`, and `sadness`; for the `language` category:
               `analytical`, `confident`, and `tentative`; for the `social` category:
               `openness_big5`, `conscientiousness_big5`, `extraversion_big5`,
               `agreeableness_big5`, and `emotional_range_big5`. The service returns
               scores for all tones of a category, regardless of their values.
        :param str tone_name: The user-visible, localized name of the tone.
        """
        self.score = score
        self.tone_id = tone_id
        self.tone_name = tone_name

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'ToneScore':
        """Initialize a ToneScore object from a json dictionary."""
        args = {}
        valid_keys = ['score', 'tone_id', 'tone_name']
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class ToneScore: '
                + ', '.join(bad_keys))
        if 'score' in _dict:
            args['score'] = _dict.get('score')
        else:
            raise ValueError(
                'Required property \'score\' not present in ToneScore JSON')
        if 'tone_id' in _dict:
            args['tone_id'] = _dict.get('tone_id')
        else:
            raise ValueError(
                'Required property \'tone_id\' not present in ToneScore JSON')
        if 'tone_name' in _dict:
            args['tone_name'] = _dict.get('tone_name')
        else:
            raise ValueError(
                'Required property \'tone_name\' not present in ToneScore JSON')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a ToneScore object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'score') and self.score is not None:
            _dict['score'] = self.score
        if hasattr(self, 'tone_id') and self.tone_id is not None:
            _dict['tone_id'] = self.tone_id
        if hasattr(self, 'tone_name') and self.tone_name is not None:
            _dict['tone_name'] = self.tone_name
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this ToneScore object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'ToneScore') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'ToneScore') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class Utterance():
    """
    An utterance for the input of the general-purpose endpoint.

    :attr str text: An utterance contributed by a user in the conversation that is
          to be analyzed. The utterance can contain multiple sentences.
    :attr str user: (optional) A string that identifies the user who contributed the
          utterance specified by the `text` parameter.
    """

    def __init__(self, text: str, *, user: str = None) -> None:
        """
        Initialize a Utterance object.

        :param str text: An utterance contributed by a user in the conversation
               that is to be analyzed. The utterance can contain multiple sentences.
        :param str user: (optional) A string that identifies the user who
               contributed the utterance specified by the `text` parameter.
        """
        self.text = text
        self.user = user

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'Utterance':
        """Initialize a Utterance object from a json dictionary."""
        args = {}
        valid_keys = ['text', 'user']
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class Utterance: '
                + ', '.join(bad_keys))
        if 'text' in _dict:
            args['text'] = _dict.get('text')
        else:
            raise ValueError(
                'Required property \'text\' not present in Utterance JSON')
        if 'user' in _dict:
            args['user'] = _dict.get('user')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a Utterance object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'text') and self.text is not None:
            _dict['text'] = self.text
        if hasattr(self, 'user') and self.user is not None:
            _dict['user'] = self.user
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this Utterance object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'Utterance') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'Utterance') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class UtteranceAnalyses():
    """
    The results of the analysis for the utterances of the input content.

    :attr List[UtteranceAnalysis] utterances_tone: An array of `UtteranceAnalysis`
          objects that provides the results for each utterance of the input.
    :attr str warning: (optional) **`2017-09-21`:** A warning message if the content
          contains more than 50 utterances. The service analyzes only the first 50
          utterances. **`2016-05-19`:** Not returned.
    """

    def __init__(self,
                 utterances_tone: List['UtteranceAnalysis'],
                 *,
                 warning: str = None) -> None:
        """
        Initialize a UtteranceAnalyses object.

        :param List[UtteranceAnalysis] utterances_tone: An array of
               `UtteranceAnalysis` objects that provides the results for each utterance of
               the input.
        :param str warning: (optional) **`2017-09-21`:** A warning message if the
               content contains more than 50 utterances. The service analyzes only the
               first 50 utterances. **`2016-05-19`:** Not returned.
        """
        self.utterances_tone = utterances_tone
        self.warning = warning

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'UtteranceAnalyses':
        """Initialize a UtteranceAnalyses object from a json dictionary."""
        args = {}
        valid_keys = ['utterances_tone', 'warning']
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class UtteranceAnalyses: '
                + ', '.join(bad_keys))
        if 'utterances_tone' in _dict:
            args['utterances_tone'] = [
                UtteranceAnalysis._from_dict(x)
                for x in (_dict.get('utterances_tone'))
            ]
        else:
            raise ValueError(
                'Required property \'utterances_tone\' not present in UtteranceAnalyses JSON'
            )
        if 'warning' in _dict:
            args['warning'] = _dict.get('warning')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a UtteranceAnalyses object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self,
                   'utterances_tone') and self.utterances_tone is not None:
            _dict['utterances_tone'] = [
                x._to_dict() for x in self.utterances_tone
            ]
        if hasattr(self, 'warning') and self.warning is not None:
            _dict['warning'] = self.warning
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this UtteranceAnalyses object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'UtteranceAnalyses') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'UtteranceAnalyses') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other


class UtteranceAnalysis():
    """
    The results of the analysis for an utterance of the input content.

    :attr int utterance_id: The unique identifier of the utterance. The first
          utterance has ID 0, and the ID of each subsequent utterance is incremented by
          one.
    :attr str utterance_text: The text of the utterance.
    :attr List[ToneChatScore] tones: An array of `ToneChatScore` objects that
          provides results for the most prevalent tones of the utterance. The array
          includes results for any tone whose score is at least 0.5. The array is empty if
          no tone has a score that meets this threshold.
    :attr str error: (optional) **`2017-09-21`:** An error message if the utterance
          contains more than 500 characters. The service does not analyze the utterance.
          **`2016-05-19`:** Not returned.
    """

    def __init__(self,
                 utterance_id: int,
                 utterance_text: str,
                 tones: List['ToneChatScore'],
                 *,
                 error: str = None) -> None:
        """
        Initialize a UtteranceAnalysis object.

        :param int utterance_id: The unique identifier of the utterance. The first
               utterance has ID 0, and the ID of each subsequent utterance is incremented
               by one.
        :param str utterance_text: The text of the utterance.
        :param List[ToneChatScore] tones: An array of `ToneChatScore` objects that
               provides results for the most prevalent tones of the utterance. The array
               includes results for any tone whose score is at least 0.5. The array is
               empty if no tone has a score that meets this threshold.
        :param str error: (optional) **`2017-09-21`:** An error message if the
               utterance contains more than 500 characters. The service does not analyze
               the utterance. **`2016-05-19`:** Not returned.
        """
        self.utterance_id = utterance_id
        self.utterance_text = utterance_text
        self.tones = tones
        self.error = error

    @classmethod
    def from_dict(cls, _dict: Dict) -> 'UtteranceAnalysis':
        """Initialize a UtteranceAnalysis object from a json dictionary."""
        args = {}
        valid_keys = ['utterance_id', 'utterance_text', 'tones', 'error']
        bad_keys = set(_dict.keys()) - set(valid_keys)
        if bad_keys:
            raise ValueError(
                'Unrecognized keys detected in dictionary for class UtteranceAnalysis: '
                + ', '.join(bad_keys))
        if 'utterance_id' in _dict:
            args['utterance_id'] = _dict.get('utterance_id')
        else:
            raise ValueError(
                'Required property \'utterance_id\' not present in UtteranceAnalysis JSON'
            )
        if 'utterance_text' in _dict:
            args['utterance_text'] = _dict.get('utterance_text')
        else:
            raise ValueError(
                'Required property \'utterance_text\' not present in UtteranceAnalysis JSON'
            )
        if 'tones' in _dict:
            args['tones'] = [
                ToneChatScore._from_dict(x) for x in (_dict.get('tones'))
            ]
        else:
            raise ValueError(
                'Required property \'tones\' not present in UtteranceAnalysis JSON'
            )
        if 'error' in _dict:
            args['error'] = _dict.get('error')
        return cls(**args)

    @classmethod
    def _from_dict(cls, _dict):
        """Initialize a UtteranceAnalysis object from a json dictionary."""
        return cls.from_dict(_dict)

    def to_dict(self) -> Dict:
        """Return a json dictionary representing this model."""
        _dict = {}
        if hasattr(self, 'utterance_id') and self.utterance_id is not None:
            _dict['utterance_id'] = self.utterance_id
        if hasattr(self, 'utterance_text') and self.utterance_text is not None:
            _dict['utterance_text'] = self.utterance_text
        if hasattr(self, 'tones') and self.tones is not None:
            _dict['tones'] = [x._to_dict() for x in self.tones]
        if hasattr(self, 'error') and self.error is not None:
            _dict['error'] = self.error
        return _dict

    def _to_dict(self):
        """Return a json dictionary representing this model."""
        return self.to_dict()

    def __str__(self) -> str:
        """Return a `str` version of this UtteranceAnalysis object."""
        return json.dumps(self._to_dict(), indent=2)

    def __eq__(self, other: 'UtteranceAnalysis') -> bool:
        """Return `true` when self and other are equal, false otherwise."""
        if not isinstance(other, self.__class__):
            return False
        return self.__dict__ == other.__dict__

    def __ne__(self, other: 'UtteranceAnalysis') -> bool:
        """Return `true` when self and other are not equal, false otherwise."""
        return not self == other



import json
from ibm_watson import DiscoveryV1
from ibm_cloud_sdk_core.authenticators import IAMAuthenticator

authenticator = IAMAuthenticator('your_api_key')
discovery = DiscoveryV1(
    version='2018-08-01',
    authenticator=authenticator)
discovery.set_service_url('https://gateway.watsonplatform.net/discovery/api')

environments = discovery.list_environments().get_result()
print(json.dumps(environments, indent=2))

news_environment_id = 'system'
print(json.dumps(news_environment_id, indent=2))

collections = discovery.list_collections(news_environment_id).get_result()
news_collections = [x for x in collections['collections']]
print(json.dumps(collections, indent=2))

configurations = discovery.list_configurations(
    environment_id=news_environment_id).get_result()
print(json.dumps(configurations, indent=2))

query_results = discovery.query(
    news_environment_id,
    news_collections[0]['collection_id'],
    filter='extracted_metadata.sha1::f5*',
    return_fields='extracted_metadata.sha1').get_result()
print(json.dumps(query_results, indent=2))

# new_environment = discovery.create_environment(name="new env", description="bogus env").get_result()
# print(new_environment)

# environment = discovery.get_environment(environment_id=new_environment['environment_id']).get_result()
# if environment['status'] == 'active':
#     writable_environment_id = new_environment['environment_id']
#     new_collection = discovery.create_collection(environment_id=writable_environment_id,
#                                                  name='Example Collection',
#                                                  description="just a test").get_result()

# print(new_collection)

# collections = discovery.list_collections(environment_id=writable_environment_id).get_result()
# print(collections)

# res = discovery.delete_collection(environment_id='<YOUR ENVIRONMENT ID>',
#                                   collection_id=new_collection['collection_id']).get_result()
# print(res)

# collections = discovery.list_collections(environment_id=writable_environment_id).get_result()
# print(collections)

# with open(os.path.join(os.getcwd(), '..','resources', 'simple.html')) as fileinfo:
#     res = discovery.add_document(environment_id=writable_environment_id,
#                                  collection_id=collections['collections'][0]['collection_id'],
#                                  file=fileinfo).get_result()
# print(res)

# res = discovery.get_collection(environment_id=writable_environment_id,
#                                collection_id=collections['collections'][0]['collection_id']).get_result()
# print(res['document_counts'])

#res = discovery.delete_environment(environment_id=writable_environment_id).get_result()
#print(res)



# coding: utf-8

# (C) Copyright IBM Corp. 2018, 2020.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ibm_watson.websocket import RecognizeCallback, RecognizeListener, AudioSource
from .speech_to_text_v1 import SpeechToTextV1
from urllib.parse import urlencode

BEARER = 'Bearer'


class SpeechToTextV1Adapter(SpeechToTextV1):

    def recognize_using_websocket(self,
                                  audio,
                                  content_type,
                                  recognize_callback,
                                  model=None,
                                  language_customization_id=None,
                                  acoustic_customization_id=None,
                                  customization_weight=None,
                                  base_model_version=None,
                                  inactivity_timeout=None,
                                  interim_results=None,
                                  keywords=None,
                                  keywords_threshold=None,
                                  max_alternatives=None,
                                  word_alternatives_threshold=None,
                                  word_confidence=None,
                                  timestamps=None,
                                  profanity_filter=None,
                                  smart_formatting=None,
                                  speaker_labels=None,
                                  http_proxy_host=None,
                                  http_proxy_port=None,
                                  customization_id=None,
                                  grammar_name=None,
                                  redaction=None,
                                  processing_metrics=None,
                                  processing_metrics_interval=None,
                                  audio_metrics=None,
                                  end_of_phrase_silence_time=None,
                                  split_transcript_at_phrase_end=None,
                                  speech_detector_sensitivity=None,
                                  background_audio_suppression=None,
                                  **kwargs):
        """
        Sends audio for speech recognition using web sockets.

        :param AudioSource audio: The audio to transcribe in the format specified by the
        `Content-Type` header.
        :param str content_type: The type of the input: audio/basic, audio/flac,
        audio/l16, audio/mp3, audio/mpeg, audio/mulaw, audio/ogg, audio/ogg;codecs=opus,
        audio/ogg;codecs=vorbis, audio/wav, audio/webm, audio/webm;codecs=opus, or
        audio/webm;codecs=vorbis.
        :param RecognizeCallback recognize_callback: The callback method for the websocket.
        :param str model: The identifier of the model that is to be used for the
        recognition request or, for the **Create a session** method, with the new session.
        :param str language_customization_id: The customization ID (GUID) of a custom
        language model that is to be used with the recognition request. The base model of
        the specified custom language model must match the model specified with the
        `model` parameter. You must make the request with service credentials created for
        the instance of the service that owns the custom model. By default, no custom
        language model is used. See [Custom
        models](https://cloud.ibm.com/docs/speech-to-text?topic=speech-to-text-input#custom).
        **Note:** Use this parameter instead of the deprecated `customization_id`
        parameter.
        :param str acoustic_customization_id: The customization ID (GUID) of a custom
        acoustic model that is to be used with the recognition request or, for the
        **Create a session** method, with the new session. The base model of the specified
        custom acoustic model must match the model specified with the `model` parameter.
        You must make the request with service credentials created for the instance of the
        service that owns the custom model. By default, no custom acoustic model is used.
        :param float customization_weight: If you specify the customization ID (GUID) of a
        custom language model with the recognition request or, for sessions, with the
        **Create a session** method, the customization weight tells the service how much
        weight to give to words from the custom language model compared to those from the
        base model for the current request.
        Specify a value between 0.0 and 1.0. Unless a different customization weight was
        specified for the custom model when it was trained, the default value is 0.3. A
        customization weight that you specify overrides a weight that was specified when
        the custom model was trained.
        The default value yields the best performance in general. Assign a higher value if
        your audio makes frequent use of OOV words from the custom model. Use caution when
        setting the weight: a higher value can improve the accuracy of phrases from the
        custom model's domain, but it can negatively affect performance on non-domain
        phrases.
        :param str base_model_version: The version of the specified base model that is to
        be used with recognition request or, for the **Create a session** method, with the
        new session. Multiple versions of a base model can exist when a model is updated
        for internal improvements. The parameter is intended primarily for use with custom
        models that have been upgraded for a new base model. The default value depends on
        whether the parameter is used with or without a custom model. For more
        information, see [Base model
        version](https://cloud.ibm.com/docs/speech-to-text?topic=speech-to-text-input#version).
        :param int inactivity_timeout: The time in seconds after which, if only silence
        (no speech) is detected in submitted audio, the connection is closed with a 400
        error. Useful for stopping audio submission from a live microphone when a user
        simply walks away. Use `-1` for infinity.
        :param List[str] keywords: (optional) An array of keyword strings to spot
        in the audio. Each keyword string can include one or more string tokens.
        Keywords are spotted only in the final results, not in interim hypotheses.
        If you specify any keywords, you must also specify a keywords threshold.
        Omit the parameter or specify an empty array if you do not need to spot
        keywords.
        You can spot a maximum of 1000 keywords with a single request. A single
        keyword can have a maximum length of 1024 characters, though the maximum
        effective length for double-byte languages might be shorter. Keywords are
        case-insensitive.
        See [Keyword
        spotting](https://cloud.ibm.com/docs/speech-to-text?topic=speech-to-text-output#keyword_spotting).
        :param float keywords_threshold: A confidence value that is the lower bound for
        spotting a keyword. A word is considered to match a keyword if its confidence is
        greater than or equal to the threshold. Specify a probability between 0 and 1
        inclusive. No keyword spotting is performed if you omit the parameter. If you
        specify a threshold, you must also specify one or more keywords.
        :param int max_alternatives: The maximum number of alternative transcripts to be
        returned. By default, a single transcription is returned.
        :param float word_alternatives_threshold: A confidence value that is the lower
        bound for identifying a hypothesis as a possible word alternative (also known as
        \"Confusion Networks\"). An alternative word is considered if its confidence is
        greater than or equal to the threshold. Specify a probability between 0 and 1
        inclusive. No alternative words are computed if you omit the parameter.
        :param bool word_confidence: If `true`, a confidence measure in the range of 0 to
        1 is returned for each word. By default, no word confidence measures are returned.
        :param bool timestamps: If `true`, time alignment is returned for each word. By
        default, no timestamps are returned.
        :param bool profanity_filter: If `true` (the default), filters profanity from all
        output except for keyword results by replacing inappropriate words with a series
        of asterisks. Set the parameter to `false` to return results with no censoring.
        Applies to US English transcription only.
        :param bool smart_formatting: If `true`, converts dates, times, series of digits
        and numbers, phone numbers, currency values, and internet addresses into more
        readable, conventional representations in the final transcript of a recognition
        request. For US English, also converts certain keyword strings to punctuation
        symbols. By default, no smart formatting is performed. Applies to US English and
        Spanish transcription only.
        :param bool speaker_labels: (optional) If `true`, the response includes
        labels that identify which words were spoken by which participants in a
        multi-person exchange. By default, the service returns no speaker labels.
        Setting `speaker_labels` to `true` forces the `timestamps` parameter to be
        `true`, regardless of whether you specify `false` for the parameter.
        **Note:** Applies to US English, German, Japanese, Korean, and Spanish
        (both broadband and narrowband models) and UK English (narrowband model)
        transcription only. To determine whether a language model supports speaker
        labels, you can also use the **Get a model** method and check that the
        attribute `speaker_labels` is set to `true`.
        See [Speaker
        labels](https://cloud.ibm.com/docs/speech-to-text?topic=speech-to-text-output#speaker_labels).
        :param str http_proxy_host: http proxy host name.
        :param str http_proxy_port: http proxy port. If not set, set to 80.
        :param str customization_id: **Deprecated.** Use the `language_customization_id`
        parameter to specify the customization ID (GUID) of a custom language model that
        is to be used with the recognition request. Do not specify both parameters with a
        request.
        :param str grammar_name: The name of a grammar that is to be used with the
        recognition request. If you specify a grammar, you must also use the
        `language_customization_id` parameter to specify the name of the custom language
        model for which the grammar is defined. The service recognizes only strings that
        are recognized by the specified grammar; it does not recognize other custom words
        from the model's words resource. See
        [Grammars](https://cloud.ibm.com/docs/speech-to-text?topic=speech-to-text-output).
        :param bool redaction: If `true`, the service redacts, or masks, numeric data from
        final transcripts. The feature redacts any number that has three or more
        consecutive digits by replacing each digit with an `X` character. It is intended
        to redact sensitive numeric data, such as credit card numbers. By default, the
        service performs no redaction.
        When you enable redaction, the service automatically enables smart formatting,
        regardless of whether you explicitly disable that feature. To ensure maximum
        security, the service also disables keyword spotting (ignores the `keywords` and
        `keywords_threshold` parameters) and returns only a single final transcript
        (forces the `max_alternatives` parameter to be `1`).
        **Note:** Applies to US English, Japanese, and Korean transcription only.
        See [Numeric
        redaction](https://cloud.ibm.com/docs/speech-to-text?topic=speech-to-text-output#redaction).
        :param bool processing_metrics: If `true`, requests processing metrics about the
        service's transcription of the input audio. The service returns processing metrics
        at the interval specified by the `processing_metrics_interval` parameter. It also
        returns processing metrics for transcription events, for example, for final and
        interim results. By default, the service returns no processing metrics.
        :param float processing_metrics_interval: Specifies the interval in real
        wall-clock seconds at which the service is to return processing metrics. The
        parameter is ignored unless the `processing_metrics` parameter is set to `true`.
        The parameter accepts a minimum value of 0.1 seconds. The level of precision is
        not restricted, so you can specify values such as 0.25 and 0.125.
        The service does not impose a maximum value. If you want to receive processing
        metrics only for transcription events instead of at periodic intervals, set the
        value to a large number. If the value is larger than the duration of the audio,
        the service returns processing metrics only for transcription events.
        :param bool audio_metrics: If `true`, requests detailed information about the
        signal characteristics of the input audio. The service returns audio metrics with
        the final transcription results. By default, the service returns no audio metrics.
        :param float end_of_phrase_silence_time: (optional) If `true`, specifies
        the duration of the pause interval at which the service splits a transcript
        into multiple final results. If the service detects pauses or extended
        silence before it reaches the end of the audio stream, its response can
        include multiple final results. Silence indicates a point at which the
        speaker pauses between spoken words or phrases.
        Specify a value for the pause interval in the range of 0.0 to 120.0.
        * A value greater than 0 specifies the interval that the service is to use
        for speech recognition.
        * A value of 0 indicates that the service is to use the default interval.
        It is equivalent to omitting the parameter.
        The default pause interval for most languages is 0.8 seconds; the default
        for Chinese is 0.6 seconds.
        See [End of phrase silence
        time](https://cloud.ibm.com/docs/speech-to-text?topic=speech-to-text-output#silence_time).
        :param bool split_transcript_at_phrase_end: (optional) If `true`, directs
        the service to split the transcript into multiple final results based on
        semantic features of the input, for example, at the conclusion of
        meaningful phrases such as sentences. The service bases its understanding
        of semantic features on the base language model that you use with a
        request. Custom language models and grammars can also influence how and
        where the service splits a transcript. By default, the service splits
        transcripts based solely on the pause interval.
        See [Split transcript at phrase
        end](https://cloud.ibm.com/docs/speech-to-text?topic=speech-to-text-output#split_transcript).
        :param float speech_detector_sensitivity: (optional) The sensitivity of
        speech activity detection that the service is to perform. Use the parameter
        to suppress word insertions from music, coughing, and other non-speech
        events. The service biases the audio it passes for speech recognition by
        evaluating the input audio against prior models of speech and non-speech
        activity.
        Specify a value between 0.0 and 1.0:
        * 0.0 suppresses all audio (no speech is transcribed).
        * 0.5 (the default) provides a reasonable compromise for the level of
        sensitivity.
        * 1.0 suppresses no audio (speech detection sensitivity is disabled).
        The values increase on a monotonic curve. See [Speech Activity
        Detection](https://cloud.ibm.com/docs/speech-to-text?topic=speech-to-text-input#detection).
        :param float background_audio_suppression: (optional) The level to which
        the service is to suppress background audio based on its volume to prevent
        it from being transcribed as speech. Use the parameter to suppress side
        conversations or background noise.
        Specify a value in the range of 0.0 to 1.0:
        * 0.0 (the default) provides no suppression (background audio suppression
        is disabled).
        * 0.5 provides a reasonable level of audio suppression for general usage.
        * 1.0 suppresses all audio (no audio is transcribed).
        The values increase on a monotonic curve. See [Speech Activity
        Detection](https://cloud.ibm.com/docs/speech-to-text?topic=speech-to-text-input#detection).
        :param dict headers: A `dict` containing the request headers
        :return: A `dict` containing the `SpeechRecognitionResults` response.
        :rtype: dict
        """
        if audio is None:
            raise ValueError('audio must be provided')
        if not isinstance(audio, AudioSource):
            raise Exception(
                'audio is not of type AudioSource. Import the class from ibm_watson.websocket'
            )
        if content_type is None:
            raise ValueError('content_type must be provided')
        if recognize_callback is None:
            raise ValueError('recognize_callback must be provided')
        if not isinstance(recognize_callback, RecognizeCallback):
            raise Exception(
                'Callback is not a derived class of RecognizeCallback')

        request = {}

        headers = {}
        if self.default_headers is not None:
            headers = self.default_headers.copy()
        if 'headers' in kwargs:
            headers.update(kwargs.get('headers'))
        request['headers'] = headers

        if self.authenticator:
            self.authenticator.authenticate(request)

        url = self.service_url.replace('https:', 'wss:')

        params = {
            'model': model,
            'customization_id': customization_id,
            'acoustic_customization_id': acoustic_customization_id,
            'base_model_version': base_model_version,
            'language_customization_id': language_customization_id
        }
        params = {k: v for k, v in params.items() if v is not None}
        url += '/v1/recognize?{0}'.format(urlencode(params))
        request['url'] = url

        options = {
            'customization_weight': customization_weight,
            'content_type': content_type,
            'inactivity_timeout': inactivity_timeout,
            'interim_results': interim_results,
            'keywords': keywords,
            'keywords_threshold': keywords_threshold,
            'max_alternatives': max_alternatives,
            'word_alternatives_threshold': word_alternatives_threshold,
            'word_confidence': word_confidence,
            'timestamps': timestamps,
            'profanity_filter': profanity_filter,
            'smart_formatting': smart_formatting,
            'speaker_labels': speaker_labels,
            'grammar_name': grammar_name,
            'redaction': redaction,
            'processing_metrics': processing_metrics,
            'processing_metrics_interval': processing_metrics_interval,
            'audio_metrics': audio_metrics,
            'end_of_phrase_silence_time': end_of_phrase_silence_time,
            'split_transcript_at_phrase_end': split_transcript_at_phrase_end,
            'speech_detector_sensitivity': speech_detector_sensitivity,
            'background_audio_suppression': background_audio_suppression
        }
        options = {k: v for k, v in options.items() if v is not None}
        request['options'] = options

        RecognizeListener(audio, request.get('options'), recognize_callback,
                          request.get('url'), request.get('headers'),
                          http_proxy_host, http_proxy_port,
                          self.disable_ssl_verification)
